from flask import Flask

# from flask_migrate import Migrate
from flask_sqlalchemy import SQLAlchemy

from auth.login import auth_bp, login_manager
# from flask_login import LoginManager
from upload.fileupload import upload_bp
from upload.stupload import stupload_bp
from upload.test import test_bp
from main import main_bp

import signal
import threading
from modbus_client import *

from config import Config


# db = SQLAlchemy()
# migrate = Migrate()


def create_app():
    app = Flask(__name__)
    app.config.from_object(Config)

    app.secret_key = "A938457D23V00"
    app.config["SESSION_TYPE"] = "filesystem"


    db = SQLAlchemy(app)

    app.register_blueprint(main_bp, url_prefix="/")
    app.register_blueprint(auth_bp, url_prefix="/auth")
    app.register_blueprint(upload_bp, url_prefix="/upload")
    app.register_blueprint(stupload_bp, url_prefix="/st")
    app.register_blueprint(test_bp, url_prefix="/test")
    # login_manager = LoginManager()
    login_manager.init_app(app)
    login_manager.login_view = 'auth.login'

    # ORM
    # db.init_app(app)
    # migrate.init_app(app, db)
    # from . import models


    # 스레드 타겟 함수 확인 출력
    print("Starting Modbus client check thread...")
    modbus_thread = threading.Thread(target=check_client)
    modbus_thread.daemon = True
    modbus_thread.start()

    # Flask 애플리케이션 실행
    print("Starting Flask application...")
    # app.run(debug=True)


    # 신호를 메인 스레드에 등록
    signal.signal(signal.SIGINT, signal.default_int_handler)
    signal.signal(signal.SIGTERM, signal.default_int_handler)




    return app
