from flask import Blueprint, redirect, url_for, request, session, flash, render_template, g
from flask_login import login_required, login_user, LoginManager
from forms import User
from werkzeug.security import generate_password_hash, check_password_hash

auth_bp = Blueprint("auth", __name__, url_prefix="/auth")
login_manager = LoginManager()

login_manager.login_view = "auth_bp.login"


user_db = {
    1: {
        'id': 'bhsystem',
        'password': generate_password_hash('bhsystem2025%')
    },

    2: {
        'id': 'test',
        'password': generate_password_hash('test2025%')
    }
}

@login_manager.user_loader
def load_user(user_id):
    return User(user_id)


@login_manager.unauthorized_handler
def unauthorized_callback():
    print("unauthorized_callback()")
    return redirect(url_for("auth.login"))


@auth_bp.route("/login", methods=["GET", "POST"])
def login():
    if request.method == "POST":
        username = request.form["username"]
        password = request.form["password"]

        user_id = None
        for uid, details in user_db.items():
            if details['id'] == username:
                user_id = uid
                break

        if user_id is None:
            flash("아이디가 존재하지 않습니다.")
            return redirect(url_for("auth.login"))
            
        if check_password_hash(user_db[user_id]['password'], password):
            user = User(id=user_id)
            login_user(user)
            print(session)
            session["username"] = username
            return redirect(url_for("main.index"))
        
        else:
            flash("비밀번호를 다시 입력해주세요...")
            return redirect(url_for("auth.login"))

    
    return render_template("auth/login.html")


@auth_bp.route("/logout")
@login_required
def logout():
    session.clear()
    return redirect(url_for("main.index"))


@auth_bp.app_context_processor
def inject_user():
    if "user_id" in session:
        user_id = session["user_id"]
        for uid, details in user_db.items():
            if uid == user_id:
                return dict(username=details['id'])
    return dict(username=None)
