from pymodbus.client import ModbusTcpClient
from pymodbus.framer import FramerType
from pymodbus.pdu.file_message import FileRecord
import os
import re
import subprocess
import crc_table

MODBUS_HOST = "127.0.0.1"
MODBUS_PORT = 65001
r_result = None

client = ModbusTcpClient(
    host=MODBUS_HOST, port=MODBUS_PORT, framer=FramerType.RTU, timeout=1000
)

def read_registers():

    # client = ModbusTcpClient(
    #     host=MODBUS_HOST, port=MODBUS_PORT, framer=FramerType.RTU, timeout=1000
    # )
    records = [FileRecord(file_number=30000, record_number=0x00, record_length=22)]

    try:
        client.connect()

        if not client.is_socket_open():
            print("서버에 연결할 수 없습니다.")
            return "연결 실패"

        rr = client.read_file_record(records=records, slave=126)

        if rr.isError():
            return "오류 발생"
        else:
            firmware_version = rr.encode()  # bytes 타입으로 encoding

            firmware_version2 = firmware_version.decode(
                "utf-8", errors="ignore"
            )  # str타입으로 decoding

            # 2글자씩 묶어서 리스트화. 이때 range(0,len(firm~~),2)였으나 1로 수정정
            grouped = [
                firmware_version2[i : i + 2]
                for i in range(1, len(firmware_version2), 2)
            ]

            # 리스트의 요소들을 역반복문을 이용해 반대로 바꿈
            rv_list = [sublist[::-1] for sublist in grouped]

            r1 = "".join(rv_list)

            result = re.sub(r"[^\x20-\x7E]", "", r1)
            print("result clean:", result)

            return result

    except Exception as e:
        print(f"Modbus 통신 오류 read_re: {e}")
        return f"Modbus 통신 오류: {e}"
    finally:
        client.close()


# ----------------------------crc_send↓-----------------------------------------
def crc_send():

    # client = ModbusTcpClient(
    #     host=MODBUS_HOST, port=MODBUS_PORT, framer=FramerType.RTU, timeout=1000
    # )

    data = 0xFFFFFFFF

    records = [FileRecord(file_number=30000, record_number=0x00, record_data=data)]

    try:
        client.connect()
        print("클라이언트 연결 되었습니다.")

        rr2 = client.write_file_record(records=records, slave=126)
        print("rr2:", rr2.records[0].record_data)

    except Exception as e:
        print(f"Modbus 통신 오류 write_re: {e}")
    finally:
        client.close()


# ----------------------------↑crc_send-----------------------------------------


def write_register(file_size, crc_value):
    print("\nwrite_register: ", file_size, ", ", crc_value)
    # client = ModbusTcpClient(
    #     host=MODBUS_HOST, port=MODBUS_PORT, framer=FramerType.RTU, timeout=1000
    # )

    data = file_size.to_bytes(4, "big") + crc_value.to_bytes(4, "big")
    print(data)

    group = [data[i : i + 2] for i in range(0, len(data), 2)]

    rv_list2 = [sublist[::-1] for sublist in group]

    result_list = b"".join(rv_list2)
    print("write_register result:", result_list)

    records = [FileRecord(file_number=30000, record_number=0x00, record_data=data)]

    try:
        client.connect()
        print("클라이언트 연결 되었습니다.")

        if not client.is_socket_open():
            print("서버에 연결할 수 없습니다.")
            return

        # 데이터 쓰기 (여기서는 레지스터 0에 값을 씀)
        rr2 = client.write_file_record(records=records, slave=126)

        print("확인2:", rr2.records[0].record_data)
        response = client.recv(2)
        print(f"response:{response}")

        if rr2.isError():
            print("오류 발생")
        else:
            print("작성 성공")

        global r_result

        if response == b"~\x00":
            subcommand = "/home/root/web-op/stop-web-op.sh"
            subprocess.Popen(
                args=[subcommand],
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                text=True,
            )
            print("subprocess실행 완료")
            r_result = "Success"
            return r_result

        elif response == b"0x40":
            r_result = "Fail System"
            return r_result
        elif response == b"0x50":
            r_result = "Fail Checksum"
            return r_result
        elif response == b"0x60":
            r_result = "Wrong File"
            return r_result
        else:
            r_result = "Time Out"
            return r_result

    except Exception as e:
        print(f"Modbus 통신 오류 write_re: {e}")
    finally:
        client.close()

    # return r_result


# plc-----------------------------------------------------------------
def plc_upload_st(file_size, crc_value, btn_value):
    print("\nwrite_register: ", file_size, ", ", crc_value)
    # client = ModbusTcpClient(
    #     host=MODBUS_HOST, port=MODBUS_PORT, framer=FramerType.RTU, timeout=1000
    # )

    data = file_size.to_bytes(4, "big") + crc_value.to_bytes(4, "big")
    print(data)

    group = [data[i : i + 2] for i in range(0, len(data), 2)]

    rv_list2 = [sublist[::-1] for sublist in group]

    result_list = b"".join(rv_list2)
    print("write_register result:", result_list)

    if btn_value == "core":
        file_number = 31110
    elif btn_value == "user":
        file_number = 31120
    else:
        file_number = 31130

    records = [
        FileRecord(file_number=file_number, record_number=0x00, record_data=data)
    ]

    try:
        client.connect()
        print("클라이언트 연결 되었습니다.")

        if not client.is_socket_open():
            print("서버에 연결할 수 없습니다.")
            return

        # 데이터 쓰기 (여기서는 레지스터 0에 값을 씀)
        rr2 = client.write_file_record(records=records, slave=126)
        print("확인 plc:", rr2.records[0].record_data)

        if rr2.isError():
            print("오류 발생")
        else:
            print("작성 성공")

    except Exception as e:
        print(f"plc upload 통신 오류{e}")

    finally:
        client.close()


# -------------------------------------------------------------------------------
def plc_write_state(type):
    # client = ModbusTcpClient(
    #     host=MODBUS_HOST, port=MODBUS_PORT, framer=FramerType.RTU, timeout=1000
    # )

    try:
        client.connect()
        print("plc write connected")

        data = type.to_bytes(2, "big")
        print("data")

        records = [FileRecord(file_number=31100, record_number=0x00, record_data=data)]
        print("records")

        rr2 = client.write_file_record(records=records, slave=126)

        print("확인 plc write records:", rr2.records[0].record_data)

        if rr2.isError():
            print("오류 발생")
        else:
            print("작성 성공")

    except Exception as e:
        return f"{e}"

    finally:
        client.close()


def plc_read_state():
    # client = ModbusTcpClient(
    #     host=MODBUS_HOST, port=MODBUS_PORT, framer=FramerType.RTU, timeout=1000
    # )

    try:
        client.connect()
        print("plc read connected")

        records = [FileRecord(file_number=31100, record_number=0x00, record_length=24)]

        rr = client.read_file_record(records=records, slave=126)

        if rr.isError():
            print("오류 발생")
            return None

        binary = rr.encode()

        result = binary[3:]

        # 2글자씩 묶어서 리스트화. 이때 range(0,len(firm~~),2)였으나 1로 수정
        grouped = [result[i : i + 2] for i in range(0, len(result), 2)]

        # 리스트의 요소들을 역반복문을 이용해 반대로 바꿈
        rv_list = [sublist[::-1] for sublist in grouped]

        r1 = b"".join(rv_list)

        hex_data = [hex(byte) for byte in r1]
        print(f"\nhex_data: {hex_data}")

        return hex_data

    except Exception as e:
        print(f"Exception occurred: {e}")
    finally:
        client.close()


# -------------------------------------------------------------------------------
def crc_calculate(filepath, init=0xFFFFFFFF):
    crc = init

    # 유효성 검사사
    if not os.path.isfile(filepath):
        raise ValueError("유효하지 않은 파일 경로입니다. ")
    with open(filepath, "rb") as f:
        while chunk := f.read(4096):
            for byte in chunk:
                crc = (
                    (crc << 8) ^ crc_table.updater_crc_table[((crc >> 24) ^ byte) & 255]
                ) & 0xFFFFFFFF

    return crc
